% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm-from-src.R
\name{dm_from_src}
\alias{dm_from_src}
\title{Load a dm from a remote data source}
\usage{
dm_from_src(src = NULL, table_names = NULL, learn_keys = NULL, ...)
}
\arguments{
\item{src}{A \pkg{dplyr} table source object or a
\code{\link[DBI:DBIConnection-class]{DBI::DBIConnection}} object is accepted.}

\item{table_names}{A character vector of the names of the tables to include.}

\item{learn_keys}{\lifecycle{experimental}

Set to \code{TRUE} to query the definition of primary and
foreign keys from the database.
Currently works only for Postgres and SQL Server databases.
The default attempts to query and issues an informative message.}

\item{...}{\lifecycle{experimental}

Additional parameters for the schema learning query.
Currently supports \code{schema} (default: \code{"public"})
and \code{table_type} (default: \code{"BASE TABLE"}) for Postgres databases.}
}
\value{
A \code{dm} object.
}
\description{
\code{dm_from_src()} creates a \link{dm} from some or all tables in a \link{src}
(a database or an environment) or which are accessible via a DBI-Connection.
For Postgres and SQL Server databases, primary and foreign keys
are imported from the database.
}
\examples{
dm_from_src_demo <- function() {
  if (!rlang::is_installed("DBI")) {
    message("Install the DBI package to run this example.")
  }

  con <- DBI::dbConnect(
    RMariaDB::MariaDB(),
    username = "guest",
    password = "relational",
    dbname = "Financial_ijs",
    host = "relational.fit.cvut.cz"
  )
  on.exit(DBI::dbDisconnect(con))

  dm_from_src(con)
}
\dontrun{
dm_from_src_demo()
}
}
